import { Logger } from '@terascope/utils';
import { Gauge, Counter, Histogram, Summary, CollectFunction } from 'prom-client';
import { Terafoundation as tf } from '@terascope/types';
export declare class PromMetrics {
    readonly metricList: tf.MetricList;
    default_labels: Record<string, string>;
    prefix: string;
    private metricExporter;
    apiConfig: tf.PromMetricsAPIConfig;
    apiRunning: boolean;
    logger: Logger;
    constructor(logger: Logger);
    /**
     * Initialize the promMetrics API and create an exporter server. Will not initialize a new API
     * if in native clustering or if the API has already been initialized.
     * @param {i.PromMetricsInitConfig} config PromMetricsInitConfig values override those
     * of terafoundation. This allows jobs to set different metrics configurations than the master.
     * @returns {Promise<boolean>} Was the API initialized
     */
    init(config: tf.PromMetricsInitConfig): Promise<boolean>;
    /**
     * [set metric to value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    set(name: string, labels: Record<string, string>, value: number): void;
    /**
     * [increment metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    inc(name: string, labels: Record<string, string>, value?: number): void;
    /**
     * [decrement metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    dec(name: string, labels: Record<string, string>, value?: number): void;
    /**
     * [observe value, used by summary metric type]
     * @param  {string} name [metric name]
     * @param  {Array<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    observe(name: string, labels: Record<string, string>, value: number): void;
    /**
     * [addGauge (define) new gauge]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {CollectFunction<Gauge>} collectFn [fn invoked when metrics endpoint scraped]
     * @return {Promise<void>}
     */
    addGauge(name: string, help: string, labelsNames: Array<string>, collectFn?: CollectFunction<Gauge>): Promise<void>;
    /**
     * [addCounter (define) new counter metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {CollectFunction<Counter>} collectFn [fn invoked when metrics endpoint scraped]
     * @return {Promise<void>}
     */
    addCounter(name: string, help: string, labelsNames: Array<string>, collectFn?: CollectFunction<Counter>): Promise<void>;
    /**
     * [addHistogram (define) new histogram metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {CollectFunction<Histogram>} collectFn [fn invoked when metrics endpoint scraped]
     * @return {Promise<void>}
     */
    addHistogram(name: string, help: string, labelsNames: Array<string>, collectFn?: CollectFunction<Histogram>, buckets?: Array<number>): Promise<void>;
    /**
     * [addSummary (define) new summary metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {CollectFunction<Summary>} collectFn [fn invoked when metrics endpoint scraped]
     * @param  {Array<number>} percentiles [metric percentiles, default[0.01, 0.1, 0.9, 0.99] ]
     * @param  {number} maxAgeSeconds [how old a bucket can be before it is reset ]
     * @param  {number} ageBuckets [how many buckets for sliding window ]
     * @return {Promise<void>}
     */
    addSummary(name: string, help: string, labelsNames: Array<string>, collectFn?: CollectFunction<Summary>, maxAgeSeconds?: number, ageBuckets?: number, percentiles?: Array<number>): Promise<void>;
    /**
     * [hasMetric check if metricList contains metric]
     * @param  {string} name [metric name]
     * @return {boolean}
     */
    hasMetric(name: string): boolean;
    /**
     * [deleteMetric delete metric from metricList]
     * @param  {string} name [metric name]
     * @return {Promise<boolean>} [was the metric successfully deleted]
     */
    deleteMetric(name: string): Promise<boolean>;
    private _createGaugeMetric;
    private _createCounterMetric;
    private _createHistogramMetric;
    private _createSummaryMetric;
    private _setPodName;
    private createAPI;
    getDefaultLabels(): Record<string, string>;
    verifyAPI(): boolean;
    resetMetrics(): void;
    shutdown(): Promise<void>;
}
//# sourceMappingURL=prom-metrics-api.d.ts.map