import fs from 'node:fs';
import path from 'node:path';
import bunyan from 'bunyan';
import { toBoolean, debugLogger, isTest, includes } from '@terascope/utils';
function getLogLevel(level) {
    // Set the same level for all logging types.
    if (typeof level === 'string') {
        return {
            console: level,
            file: level
        };
    }
    // Otherwise there may be a list of separate settins for each type.
    return level.reduce((prev, curr) => {
        Object.assign(prev, curr);
        return prev;
    }, {});
}
export function createRootLogger(context) {
    const useDebugLogger = (toBoolean(process.env.USE_DEBUG_LOGGER || isTest))
        && !toBoolean(process.env.TESTING_LOG_LEVEL);
    const filename = context.name;
    const name = context.assignment || filename;
    const foundationConfig = context.sysconfig.terafoundation;
    const logLevel = getLogLevel(foundationConfig.log_level);
    if (useDebugLogger) {
        return debugLogger(`${filename}:${name}`);
    }
    const streamConfig = [];
    const { environment = 'development' } = foundationConfig;
    // Setup console logging. Always turned on for development but off by
    // default for production.
    if (environment === 'development' || includes(foundationConfig.logging, 'console')) {
        const level = logLevel.console ? logLevel.console : 'info';
        streamConfig.push({ stream: process.stdout, level });
    }
    // Setup logging to files.
    if (includes(foundationConfig.logging, 'file')) {
        const configPath = foundationConfig.log_path || './logs';
        // remove whitespace
        const logfile = filename.trim();
        try {
            // Verify the path is a directory by resolving any symlinks
            const pathCheck = fs.lstatSync(fs.realpathSync(configPath));
            if (pathCheck.isDirectory()) {
                const level = logLevel.file ? logLevel.file : 'info';
                streamConfig.push({ path: path.join(configPath, `${logfile}.log`), level });
            }
            else {
                // This is error is just caught by the catch block below.
                throw new Error(`${configPath} is not a directory`);
            }
        }
        catch (e) {
            throw new Error(`Could not write to log_path: ${configPath}`);
        }
    }
    const loggerConfig = {
        name: filename,
        streams: streamConfig,
        assignment: context.assignment,
    };
    const logger = bunyan.createLogger(loggerConfig);
    logger.flush = async () => true;
    return logger;
}
//# sourceMappingURL=utils.js.map