import { isString, get, isFunction, getTypeOf } from '@terascope/utils';
import { nanoid } from 'nanoid';
import { CoreContext } from './core-context.js';
import validateConfigs from './validate-configs.js';
import { PromMetrics } from './api/prom-metrics/prom-metrics-api.js';
function getKey(opts) {
    const { type, endpoint = 'default' } = opts;
    if (!isString(type))
        throw new Error('A type must be specified when registering a Client');
    return `${type}:${endpoint}`;
}
function setConnectorConfig(sysconfig, opts, config, override = true) {
    const { type, endpoint = 'default' } = opts;
    const { connectors } = sysconfig.terafoundation;
    if (connectors[type] == null)
        connectors[type] = {};
    if (connectors[type][endpoint] == null) {
        connectors[type][endpoint] = config;
    }
    else if (override) {
        connectors[type][endpoint] = config;
    }
    return connectors[type][endpoint];
}
const _cachedClients = new WeakMap();
const _createClientFns = new WeakMap();
function getDefaultSysconfig(options) {
    return {
        terafoundation: {
            connectors: {
                'elasticsearch-next': {
                    default: {},
                },
                s3: {
                    default: {}
                }
            },
            ...get(options.sysconfig, 'terafoundation', {})
        },
        ...options.sysconfig
    };
}
export class TestContext extends CoreContext {
    constructor(config, cluster, sysconfig) {
        super(config, cluster, sysconfig);
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const ctx = this;
        _cachedClients.set(this, {});
        _createClientFns.set(this, {});
        this.apis.foundation.createClient = async (opts) => {
            const { cached } = opts;
            const cachedClients = _cachedClients.get(ctx) || {};
            const key = getKey(opts);
            if (cached && cachedClients[key] != null) {
                return cachedClients[key];
            }
            const clientFns = _createClientFns.get(ctx) || {};
            const create = clientFns[key];
            if (!create)
                throw new Error(`No client was found for connection "${key}"`);
            if (!isFunction(create)) {
                const actual = getTypeOf(create);
                throw new Error(`Registered Client for connection "${key}" is not a function, got ${actual}`);
            }
            const connectorConfig = setConnectorConfig(ctx.sysconfig, opts, {}, false);
            const client = await create(connectorConfig, ctx.logger, opts);
            cachedClients[key] = client;
            _cachedClients.set(ctx, cachedClients);
            return client;
        };
        this.apis.foundation.promMetrics = new PromMetrics(ctx.logger);
        this.apis.setTestClients = (clients = []) => {
            clients.forEach((clientConfig) => {
                const { createClient, config: connectionConfig = {} } = clientConfig;
                const createFN = createClient;
                const clientFns = _createClientFns.get(ctx) || {};
                const key = getKey(clientConfig);
                if (!isFunction(createFN)) {
                    const actual = getTypeOf(createFN);
                    throw new Error(`Test Client for connection "${key}" is not a function, got ${actual}`);
                }
                ctx.logger.trace(`Setting test client for connection "${key}"`, connectionConfig);
                clientFns[key] = createFN;
                _createClientFns.set(ctx, clientFns);
                const cachedClients = _cachedClients.get(ctx) || {};
                delete cachedClients[key];
                _cachedClients.set(ctx, cachedClients);
                setConnectorConfig(ctx.sysconfig, clientConfig, connectionConfig, true);
            });
        };
        this.apis.getTestClients = () => {
            const cachedClients = _cachedClients.get(ctx) || {};
            const clients = {};
            Object.keys(cachedClients).forEach((key) => {
                const [type, endpoint] = key.split(':', 2);
                if (clients[type] == null) {
                    clients[type] = {};
                }
                clients[type][endpoint] = cachedClients[key];
            });
            return clients;
        };
    }
    static async createContext(options = {}) {
        const defaultSysconfig = getDefaultSysconfig(options);
        const config = {
            name: options.name || 'test-context',
        };
        const cluster = {
            isMaster: false,
            worker: {
                id: nanoid(8),
            }
        };
        const sysConfig = await validateConfigs(cluster, config, defaultSysconfig);
        const context = new TestContext(config, cluster, sysConfig);
        if (options.clients) {
            context.apis.setTestClients(options.clients);
        }
        return context;
    }
}
//# sourceMappingURL=test-context.js.map