import os from 'node:os';
import convict from 'convict';
import { TSError, isFunction, isPlainObject, isEmpty, concat, pMap, cloneDeep } from '@terascope/utils';
// @ts-expect-error no types
import convict_format_with_validator from 'convict-format-with-validator';
// @ts-expect-error no types
import convict_format_with_moment from 'convict-format-with-moment';
import { getConnectorSchemaAndValFn } from './connector-utils.js';
import { foundationSchema } from './schema.js';
convict.addFormats(convict_format_with_validator);
convict.addFormats(convict_format_with_moment);
function validateConfig(cluster, schema, namespaceConfig) {
    try {
        const config = convict(schema || {});
        config.load(namespaceConfig);
        if (cluster.isMaster) {
            config.validate({
                // IMPORTANT: changing this will break things
                // FIXME
                // false is deprecated and will be removed in ^5.0.0
                // must be warn or strict
                allowed: true,
            });
        }
        return config.getProperties();
    }
    catch (err) {
        throw new TSError(err, { reason: 'Error validating configuration' });
    }
}
async function extractSchema(fn, sysconfig) {
    if (isFunction(fn)) {
        const result = await fn(sysconfig);
        if (result.schema) {
            return result.schema;
        }
        return result;
    }
    if (isPlainObject(fn)) {
        return fn;
    }
    return {};
}
function extractValidatorFn(fn, sysconfig) {
    if (isFunction(fn)) {
        const result = fn(sysconfig);
        if (result.validatorFn) {
            return result.validatorFn;
        }
    }
    if (isPlainObject(fn)) {
        return fn.validatorFn;
    }
    return undefined;
}
/**
 * @param cluster the nodejs cluster metadata
 * @param config the config object passed to the library terafoundation
 * @param sysconfig unvalidated sysconfig
*/
export default async function validateConfigs(cluster, config, sysconfig) {
    if (!isPlainObject(config) || isEmpty(config)) {
        throw new Error('Terafoundation requires a valid application configuration');
    }
    if (!isPlainObject(sysconfig) || isEmpty(sysconfig)) {
        throw new Error('Terafoundation requires a valid system configuration');
    }
    const listOfValidations = {};
    const schema = await extractSchema(config.config_schema, sysconfig);
    schema.terafoundation = foundationSchema();
    const result = {};
    if (config.schema_formats) {
        config.schema_formats.forEach((format) => {
            convict.addFormat(format);
        });
    }
    const schemaKeys = concat(Object.keys(schema), Object.keys(sysconfig));
    for (const schemaKey of schemaKeys) {
        const subSchema = schema[schemaKey] || {};
        const subConfig = sysconfig[schemaKey] || {};
        const validatedConfig = validateConfig(cluster, subSchema, subConfig);
        result[schemaKey] = validatedConfig;
        if (schemaKey === 'terafoundation') {
            result[schemaKey].connectors = {};
            const connectors = subConfig.connectors || {};
            const connectorList = Object.entries(connectors);
            await pMap(connectorList, async ([connector, connectorConfig]) => {
                const { schema: connectorSchema, validatorFn: connValidatorFn } = await getConnectorSchemaAndValFn(connector);
                result[schemaKey].connectors[connector] = {};
                for (const [connection, connectionConfig] of Object.entries(connectorConfig)) {
                    const validatedConnConfig = validateConfig(cluster, connectorSchema, connectionConfig);
                    result[schemaKey].connectors[connector][connection] = validatedConnConfig;
                    listOfValidations[`${connector}:${connection}`] = {
                        validatorFn: connValidatorFn,
                        config: validatedConnConfig,
                        connector: true
                    };
                }
            });
            listOfValidations[schemaKey] = {
                config: validatedConfig
            };
        }
        else {
            listOfValidations[schemaKey] = {
                validatorFn: extractValidatorFn(config.config_schema, sysconfig),
                config: validatedConfig
            };
        }
    }
    // Annotate the config with some information about this instance.
    const hostname = os.hostname();
    if (process.env.POD_IP) {
        result._nodeName = process.env.POD_IP;
    }
    else if (cluster.worker) {
        result._nodeName = `${hostname}.${cluster.worker.id}`;
    }
    else {
        result._nodeName = hostname;
    }
    // Cross-field validation
    for (const entry of Object.entries(listOfValidations)) {
        const [name, validatorObj] = entry;
        if (validatorObj.validatorFn) {
            try {
                validatorObj.validatorFn(cloneDeep(validatorObj.config), cloneDeep(result));
            }
            catch (err) {
                throw new TSError(`Cross-field validation failed for ${validatorObj.connector ? 'connector ' : ''}'${name}': ${err}`);
            }
        }
    }
    return result;
}
//# sourceMappingURL=validate-configs.js.map